import { AngularFireAuth } from '@angular/fire/auth';
import { AngularFirestore } from '@angular/fire/firestore';
import { auth, User, firestore } from 'firebase/app';
import 'firebase/auth';
import { Observable } from 'rxjs';
import { Store, UpdateStateCallback } from '@datorama/akita';
import { FireAuthState } from './auth.model';
import { WriteOptions } from '../utils/types';
export declare const fireAuthProviders: readonly ["github", "google", "microsoft", "facebook", "twitter", "email"];
export declare type FireProvider = (typeof fireAuthProviders)[number];
/** Verify if provider is part of the list of Authentication provider provided by Firebase Auth */
export declare function isFireAuthProvider(provider: string): provider is FireProvider;
/**
 * Get the custom claims of a user. If no key is provided, return the whole claims object
 * @param user The user object returned by Firebase Auth
 * @param roles Keys of the custom claims inside the claim objet
 */
export declare function getCustomClaims(user: User, roles?: string | string[]): Promise<Record<string, any>>;
/**
 * Get the Authentication Provider based on its name
 * @param provider string literal representing the name of the provider
 */
export declare function getAuthProvider(provider: FireProvider): auth.EmailAuthProvider | auth.FacebookAuthProvider | auth.GithubAuthProvider | auth.GoogleAuthProvider | auth.OAuthProvider | auth.TwitterAuthProvider;
export declare class FireAuthService<S extends FireAuthState> {
    protected store: Store<S>;
    private collection;
    protected collectionPath: string;
    protected fireAuth: AngularFireAuth;
    protected db: AngularFirestore;
    /** Triggered when the profile has been created */
    protected onCreate?(profile: S['profile'], options: WriteOptions): any;
    /** Triggered when the profile has been updated */
    protected onUpdate?(profile: S['profile'], options: WriteOptions): any;
    /** Triggered when the profile has been deleted */
    protected onDelete?(options: WriteOptions): any;
    /** Triggered when user signin for the first time or signup with email & password */
    protected onSignup?(user: auth.UserCredential, options: WriteOptions): any;
    /** Triggered when a user signin, except for the first time @see onSignup */
    protected onSignin?(user: auth.UserCredential): any;
    /** Triggered when a user signout */
    protected onSignout?(): any;
    constructor(store: Store<S>, db?: AngularFirestore, fireAuth?: AngularFireAuth);
    readonly idKey: any;
    /**
     * Select the profile in the Firestore
     * @note can be override to point to a different place
     */
    protected selectProfile(user: User): Observable<S['profile']>;
    /**
     * Select the roles for this user. Can be in custom claims or in a Firestore collection
     * @param user The user given by FireAuth
     * @see getCustomClaims to get the custom claims out of the user
     * @note Can be overrided
     */
    protected selectRoles(user: User): Promise<S['roles']> | Observable<S['roles']>;
    /**
     * Function triggered when getting data from firestore
     * @note should be overrided
     */
    protected formatFromFirestore(user: any): S['profile'];
    /**
     * Function triggered when adding/updating data to firestore
     * @note should be overrided
     */
    protected formatToFirestore(user: S['profile']): any;
    /**
     * Function triggered when transforming a user into a profile
     * @param user The user object from FireAuth
     * @param ctx The context given on signup
     * @note Should be override
     */
    protected createProfile(user: User, ctx?: any): Promise<Partial<S['profile']>> | Partial<S['profile']>;
    /** The current sign-in user (or null) */
    readonly user: User;
    /** The path to the profile in firestore */
    readonly path: any;
    /** Start listening on User */
    sync(): Observable<any[]>;
    /**
     * @description Delete user from authentication service and database
     * WARNING This is security sensitive operation
     */
    delete(options?: WriteOptions): Promise<void>;
    /** Update the current profile of the authenticated user */
    update(profile: Partial<S['profile']> | UpdateStateCallback<S['profile']>, options?: WriteOptions): Promise<void | firestore.Transaction>;
    /** Create a user based on email and password */
    signup(email: string, password: string, options?: WriteOptions): Promise<auth.UserCredential>;
    /** Signin with email & passwor, provider or custom token */
    signin(email: string, password: string): Promise<auth.UserCredential>;
    signin(provider?: FireProvider): Promise<auth.UserCredential>;
    signin(token: string): Promise<auth.UserCredential>;
    /** Signs out the current user and clear the store */
    signOut(): Promise<void>;
}
