import { AngularFirestoreCollection, AngularFirestore, DocumentChangeAction, QueryFn, QueryGroupFn } from '@angular/fire/firestore';
import { EntityStore, EntityState, UpdateStateCallback, ActiveState, getEntityType } from '@datorama/akita';
import { firestore } from 'firebase/app';
import 'firebase/firestore';
import { CollectionOptions } from './collection.config';
import { WriteOptions, SyncOptions, PathParams } from '../utils/types';
import { Observable } from 'rxjs';
export declare type CollectionState<E = any> = EntityState<E, string> & ActiveState<string>;
export declare type orObservable<Input, Output> = Input extends Observable<infer I> ? Observable<Output> : Output;
export declare type DocOptions = {
    path: string;
} | {
    id: string;
};
export declare type GetRefs<idOrQuery> = idOrQuery extends (infer I)[] ? firestore.DocumentReference[] : idOrQuery extends string ? firestore.DocumentReference : firestore.CollectionReference;
export declare class CollectionService<S extends EntityState<any, string>> {
    protected store?: EntityStore<S>;
    private collectionPath?;
    private idsToListen;
    protected db: AngularFirestore;
    protected onCreate?(entity: getEntityType<S>, options: WriteOptions): any;
    protected onUpdate?(entity: Partial<getEntityType<S>>, options: WriteOptions): any;
    protected onDelete?(id: string, options: WriteOptions): any;
    constructor(store?: EntityStore<S>, collectionPath?: string, db?: AngularFirestore);
    protected getPath(options: PathParams): string;
    readonly idKey: string;
    /** The path to the collection in Firestore */
    readonly path: string;
    /** A snapshot of the path */
    readonly currentPath: string;
    /**
     * The Angular Fire collection
     * @notice If path is an observable, it becomes an observable.
     */
    readonly collection: AngularFirestoreCollection<getEntityType<S>>;
    /** @deprecated Please use @see formatToFirestore */
    protected preFormat<E extends getEntityType<S>>(document: Readonly<Partial<E>>): E;
    /**
     * Function triggered when adding/updating data to firestore
     * @note should be overrided
     */
    protected formatToFirestore(entity: Partial<getEntityType<S>>): any;
    /** The config given by the `CollectonConfig` */
    readonly config: CollectionOptions;
    /**
     * Sync a store with a collection query of Firestore
     * @param path Path of the collection in Firestore
     * @param queryFn A query function to filter document of the collection
     * @param syncOptions Options about the store to sync with Firestore
     * @example
     * service.syncCollection({ storeName: 'movies-latest', loading: false }).subscribe();
     */
    syncCollection(syncOptions?: Partial<SyncOptions>): Observable<DocumentChangeAction<getEntityType<S>>[]>;
    /**
     * @example
     * service.syncCollection(activePath$).subscribe();
     */
    syncCollection(path: string, syncOptions?: Partial<SyncOptions>): Observable<DocumentChangeAction<getEntityType<S>>[]>;
    /**
     * @example
     * service.syncCollection(ref => ref.limit(10), { storeName: 'movie-latest'}).subscribe();
     */
    syncCollection(query: QueryFn, syncOptions?: Partial<SyncOptions>): Observable<DocumentChangeAction<getEntityType<S>>[]>;
    /**
     * @example
     * service.syncCollection('movies', ref => ref.limit(10), { loading: false }).subscribe();
     */
    syncCollection(path: string, queryFn?: QueryFn, syncOptions?: Partial<SyncOptions>): Observable<DocumentChangeAction<getEntityType<S>>[]>;
    /**
     * Sync a store with a collection group
     * @param collectionId An id of
     * @param queryFn A query function to filter document of the collection
     * @param syncOptions Options about the store to sync with Firestore
     */
    syncCollectionGroup(syncOptions?: Partial<SyncOptions>): Observable<DocumentChangeAction<getEntityType<S>>[]>;
    syncCollectionGroup(queryGroupFn?: QueryGroupFn): Observable<DocumentChangeAction<getEntityType<S>>[]>;
    syncCollectionGroup(collectionId: string, syncOptions?: Partial<SyncOptions>): Observable<DocumentChangeAction<getEntityType<S>>[]>;
    syncCollectionGroup(collectionId: string, queryGroupFn?: QueryGroupFn, syncOptions?: Partial<SyncOptions>): Observable<DocumentChangeAction<getEntityType<S>>[]>;
    /**
     * Sync the store with several documents
     * @param ids An array of ids or an observable of ids
     * @param syncOptions Options on the store to sync to
     */
    syncManyDocs(ids: string[], syncOptions?: Partial<SyncOptions>): any;
    /** @deprecated Use syncManyDocs inside a switchMap instead of passing an observable */
    syncManyDocs(ids$: Observable<string[]>, syncOptions?: Partial<SyncOptions>): any;
    /**
     * Stay in sync with one document
     * @param docOptions An object with EITHER `id` OR `path`.
     * @note We need to use id and path because there is no way to differentiate them.
     * @param syncOptions Options on the store to sync to
     */
    syncDoc(docOptions: DocOptions, syncOptions?: Partial<SyncOptions>): Observable<getEntityType<S>>;
    /**
     * Stay in sync with the active entity and set it active in the store
     * @param options A list of ids or An object with EITHER `id` OR `path`.
     * @note We need to use id and path because there is no way to differentiate them.
     * @param syncOptions Options on the store to sync to
     */
    syncActive(options: S['active'] extends any[] ? string[] : DocOptions, syncOptions?: Partial<SyncOptions>): S['active'] extends any[] ? Observable<getEntityType<S>[]> : Observable<getEntityType<S>>;
    /** Return the reference of the document(s) or collection */
    getRef(options?: Partial<SyncOptions>): Promise<firestore.CollectionReference<getEntityType<S>>>;
    getRef(ids?: string[], options?: Partial<SyncOptions>): Promise<firestore.DocumentReference<getEntityType<S>>[]>;
    getRef(query?: QueryFn, options?: Partial<SyncOptions>): Promise<firestore.CollectionReference<getEntityType<S>>>;
    getRef(id?: string, options?: Partial<SyncOptions>): Promise<firestore.DocumentReference<getEntityType<S>>>;
    /** Return the current value of the path from Firestore */
    getValue(options?: Partial<SyncOptions>): Promise<getEntityType<S>[]>;
    getValue(ids?: string[], options?: Partial<SyncOptions>): Promise<getEntityType<S>[]>;
    getValue(query?: QueryFn, options?: Partial<SyncOptions>): Promise<getEntityType<S>[]>;
    getValue(id?: string, options?: Partial<SyncOptions>): Promise<getEntityType<S>>;
    /**
     * Add a document or a list of document to Firestore
     * @param docs A document or a list of document
     * @param write batch or transaction to run the operation into
     */
    add<D extends (Partial<getEntityType<S>> | Partial<getEntityType<S>>[])>(documents: D, options?: WriteOptions): Promise<D extends (infer I)[] ? string[] : string>;
    /**
     * Remove one or several document from Firestore
     * @param id A unique or list of id representing the document
     * @param write batch or transaction to run the operation into
     */
    remove(id: string | string[], options?: WriteOptions): Promise<void>;
    /** Remove all document of the collection */
    removeAll(options?: WriteOptions): Promise<void>;
    /**
     * Update one or several document in Firestore
     */
    update(entity: Partial<getEntityType<S>> | Partial<getEntityType<S>>[], options?: WriteOptions): Promise<void>;
    update(ids: string | string[], newStateFn: UpdateStateCallback<getEntityType<S>> | Partial<getEntityType<S>>, options?: WriteOptions): Promise<void>;
}
